<?php
// $Id: user_terms_handler_field_term_user_tid.inc,v 1.1.2.3 2010/01/16 12:57:06 joachim Exp $

/**
 * Field handler for terms. 
 *
 * Heavily based on views_handler_field_term_node_tid, but we don't inherit 
 * from that because of too many differences.
 */
class user_terms_handler_field_term_user_tid extends views_handler_field_prerender_list {
  function init(&$view, $options) {
    parent::init($view, $options);
    
    // Add the uid field to we're guaranteed it when this field is on a view 
    // whose base is other than user.
    $this->additional_fields['uid'] = array(
      'table' => 'users', 
      'field' => 'uid'
    );    
  }

  function option_definition() {
    $options = parent::option_definition();
    $options['link_to_taxonomy'] = array('default' => TRUE);

    $user_terms_vocabulary = variable_get('user_terms_vocabulary', -1);
    $vocabularies = taxonomy_get_vocabularies();
    $options['vids'] = array(
      'default' => array(
        $user_terms_vocabulary => $user_terms_vocabulary,
      ),
    );
    
    return $options;
  }

  /**
   * Provide "link to term" option.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['link_to_taxonomy'] = array(
      '#title' => t('Link this field to its term page'),
      '#type' => 'checkbox',
      '#default_value' => !empty($this->options['link_to_taxonomy']),
    );
  }

  /**
   * Fake field: don't query.
   */
  function query() {
    $this->add_additional_fields();
  }

  /**
   * Loads all terms and builds an array of items with the uid as index.
   */
  function pre_render($values) {
    $this->field_alias = $this->aliases['uid'];

    foreach ($values as $result) {
      $uids[$result->{$this->aliases['uid']}] = $result->{$this->aliases['uid']};
    }

    if ($uids) {
      $tids_string = implode(',', array_filter($uids));
      $result = db_query("SELECT td.*, tu.uid, v.name as vocabulary FROM {term_data} td INNER JOIN {term_user} tu ON td.tid = tu.tid INNER JOIN {vocabulary} v ON v.vid = td.vid WHERE tu.uid IN ($tids_string) ORDER BY td.weight, td.name");
  
      while ($term = db_fetch_object($result)) {
        $this->items[$term->uid][$term->tid]['name'] = check_plain($term->name); 
        $this->items[$term->uid][$term->tid]['tid'] = $term->tid;
        $this->items[$term->uid][$term->tid]['vid'] = $term->vid;
        $this->items[$term->uid][$term->tid]['vocabulary'] = check_plain($term->vocabulary);
          
        if (!empty($this->options['link_to_taxonomy'])) {
          $this->items[$term->uid][$term->tid]['make_link'] = TRUE;
          $this->items[$term->uid][$term->tid]['path'] = taxonomy_term_path($term);
        }
      }
    }
  }
  
  function render_item($count, $item) {
    return $item['name'];
  }  

  function document_self_tokens(&$tokens) {
    $tokens['[' . $this->options['id'] . '-tid' . ']'] = t('The taxonomy term ID for the term.');
    $tokens['[' . $this->options['id'] . '-name' . ']'] = t('The taxonomy term name for the term.');
    $tokens['[' . $this->options['id'] . '-vid' . ']'] = t('The vocabulary ID for the vocabulary the term belongs to.');
    $tokens['[' . $this->options['id'] . '-vocabulary' . ']'] = t('The name for the vocabulary the term belongs to.');
  }

  function add_self_tokens(&$tokens, $item) {
    $tokens['[' . $this->options['id'] . '-tid' . ']'] = $item['tid'];
    $tokens['[' . $this->options['id'] . '-name' . ']'] = $item['name'];
    $tokens['[' . $this->options['id'] . '-vid' . ']'] = $item['vid'];
    $tokens['[' . $this->options['id'] . '-vocabulary' . ']'] = $item['vocabulary'];
  }
}
